@@ -6,7 +6,6 @@ from collections import OrderedDict
 from os import remove
 from os.path import splitext, join, exists
 from posixpath import basename
-from tempfile import gettempdir
 from typing import Optional, Dict, Iterator, Union, List, Any, Tuple
 
 import requests
@@ -16,6 +15,7 @@ from six.moves.urllib.parse import urlsplit, urlunsplit, parse_qsl, urlencode
 from tabulator.exceptions import TabulatorException
 
 from hdx.utilities import raisefrom
+from hdx.utilities.path import get_temp_dir
 from hdx.utilities.session import get_session
 
 logger = logging.getLogger(__name__)
@@ -36,6 +36,7 @@ class Download(object):
         extra_params_dict (Dict[str, str]): Extra parameters to put on end of url as a dictionary OR
         extra_params_json (str): Path to JSON file containing extra parameters to put on end of url OR
         extra_params_yaml (str): Path to YAML file containing extra parameters to put on end of url
+        extra_params_lookup (str): Lookup key for parameters. If not given assumes parameters are at root of the dict.
         status_forcelist (List[int]): HTTP statuses for which to force retry
         method_whitelist (iterable): HTTP methods for which to force retry. Defaults t0 frozenset(['GET']).
     """
@@ -70,9 +71,11 @@ class Download(object):
         self.session.close()
 
     def __enter__(self):
+        # type: () -> 'Download'
         return self
 
     def __exit__(self, exc_type, exc_value, traceback):
+        # type: (Any, Any, Any) -> None
         self.close()
 
     @staticmethod
@@ -95,7 +98,7 @@ class Download(object):
             filename = basename(urlpath)
         filename, extension = splitext(filename)
         if not folder:
-            folder = gettempdir()
+            folder = get_temp_dir()
         path = join(folder, '%s%s' % (filename, extension))
         if overwrite:
             try:
@@ -110,9 +113,12 @@ class Download(object):
         return path
 
     def get_full_url(self, url):
-        # type: () -> str
+        # type: (str) -> str
         """Get full url including any additional parameters
 
+        Args:
+            url (str): URL for which to get full url
+
         Returns:
             str: Full url including any additional parameters
         """
@@ -276,9 +282,19 @@ class Download(object):
         """
         return self.setup(url, stream=False, post=post, parameters=parameters, timeout=timeout)
 
+    def get_json(self):
+        # type: () -> Any
+        """Get JSON content of download
+
+        Returns:
+            Any: JSON content of download
+
+        """
+        return self.response.json(object_pairs_hook=OrderedDict)
+
     def get_tabular_stream(self, url, **kwargs):
         # type: (str, Any) -> tabulator.Stream
-        """Get iterator for reading rows from tabular data. Each row is returned as a dictionary.
+        """Get Tabulator stream.
 
         Args:
             url (str): URL to download
