@@ -59,16 +59,33 @@ class TestDownloader:
         path = Download.get_path_for_url(fixtureurl, downloaderfolder, filename)
         assert abspath(path) == abspath(join(downloaderfolder, filename))
 
-    def test_init(self, downloaderfolder):
+    def test_init(self, monkeypatch, downloaderfolder):
         basicauthfile = join(downloaderfolder, 'basicauth.txt')
+        with Download(auth=('u', 'p')) as downloader:
+            assert downloader.session.auth == ('u', 'p')
+        basicauth = 'Basic dXNlcjpwYXNz'
+        with Download(basic_auth=basicauth) as downloader:
+            assert downloader.session.auth == ('user', 'pass')
         with Download(basic_auth_file=basicauthfile) as downloader:
             assert downloader.session.auth == ('testuser', 'testpass')
+        extraparamsyamltree = join(downloaderfolder, 'extra_params_tree.yml')
+        with Download(basic_auth='Basic dXNlcjpwYXNz', extra_params_yaml=extraparamsyamltree,
+                      extra_params_lookup='mykey') as downloader:
+            assert downloader.session.auth == ('user', 'pass')
+        monkeypatch.setenv('BASIC_AUTH', basicauth)
+        with Download(basic_auth='12345') as downloader:
+            assert downloader.session.auth == ('user', 'pass')
+        monkeypatch.delenv('BASIC_AUTH')
         with pytest.raises(SessionError):
             Download(auth=('u', 'p'), basic_auth='Basic xxxxxxxxxxxxxxxx')
         with pytest.raises(SessionError):
             Download(auth=('u', 'p'), basic_auth_file=join('lala', 'lala.txt'))
         with pytest.raises(SessionError):
             Download(basic_auth='Basic dXNlcjpwYXNz', basic_auth_file=join('lala', 'lala.txt'))
+        with pytest.raises(SessionError):
+            Download(auth=('u', 'p'), extra_params_yaml=extraparamsyamltree, extra_params_lookup='mykey')
+        with pytest.raises(SessionError):
+            Download(basic_auth_file=basicauthfile, extra_params_yaml=extraparamsyamltree, extra_params_lookup='mykey')
         with pytest.raises(IOError):
             Download(basic_auth_file='NOTEXIST')
         extraparamsjson = join(downloaderfolder, 'extra_params.json')
@@ -87,10 +104,29 @@ class TestDownloader:
             assert 'param1=value1' in full_url
             assert 'param2=value+2' in full_url
             assert 'param3=10' in full_url
+        with Download(extra_params_yaml=extraparamsyamltree, extra_params_lookup='mykey') as downloader:
+            assert downloader.session.auth == ('testuser', 'testpass')
+            full_url = downloader.get_full_url(test_url)
+            assert 'param1=value+1' in full_url
+            assert 'param2=value2' in full_url
+            assert 'param3=11' in full_url
+            assert 'basic_auth' not in full_url
+        monkeypatch.setenv('EXTRA_PARAMS', 'param1=value2+3,param2=value3+7')
+        with Download(extra_params_yaml=extraparamsyamltree, extra_params_lookup='mykey') as downloader:
+            assert downloader.session.auth is None
+            full_url = downloader.get_full_url(test_url)
+            assert 'param1=value2%2B3' in full_url
+            assert 'param2=value3%2B7' in full_url
+            assert 'param3=11' not in full_url
+            assert 'basic_auth' not in full_url
+        monkeypatch.delenv('EXTRA_PARAMS')
         with pytest.raises(SessionError):
             Download(extra_params_dict={'key1': 'val1'}, extra_params_json=extraparamsjson)
         with pytest.raises(SessionError):
             Download(extra_params_dict={'key1': 'val1'}, extra_params_yaml=extraparamsyaml)
+        with pytest.raises(SessionError):
+            Download(extra_params_dict={'key1': 'val1'}, extra_params_yaml=extraparamsyamltree,
+                     extra_params_lookup='mykey')
         with pytest.raises(IOError):
             Download(extra_params_json='NOTEXIST')
 
@@ -115,13 +151,13 @@ class TestDownloader:
         with Download() as downloader:
             downloader.setup(postfixtureurl, post=True)
             headers = downloader.response.headers
-            assert bool(re.match(r'27\d', headers['Content-Length'])) is True
+            assert bool(re.match(r'3[56]\d', headers['Content-Length'])) is True
             downloader.setup('%s?id=10&lala=a' % getfixtureurl, post=False,
                              parameters=OrderedDict([('b', '4'), ('d', '3')]))
-            assert downloader.response.json()['args'] == {'id': '10', 'lala': 'a', 'b': '4', 'd': '3'}
+            assert downloader.get_json()['args'] == OrderedDict([('b', '4'), ('d', '3'), ('id', '10'), ('lala', 'a')])
             downloader.setup('%s?id=3&lala=b' % postfixtureurl, post=True,
                              parameters=OrderedDict([('a', '3'), ('c', '2')]))
-            assert downloader.response.json()['form'] == {'id': '3', 'lala': 'b', 'a': '3', 'c': '2'}
+            assert downloader.get_json()['form'] == OrderedDict([('a', '3'), ('c', '2'), ('id', '3'), ('lala', 'b')])
 
     def test_hash_stream(self, fixtureurl):
         with Download() as downloader:
@@ -151,10 +187,10 @@ class TestDownloader:
             fpath = abspath(f)
             with open(fpath, 'rt') as fi:
                 text = fi.read()
-                assert '"id":"10"' in text
-                assert '"lala":"a"' in text
-                assert '"b":"4"' in text
-                assert '"d":"3"' in text
+                assert '"id": "10"' in text
+                assert '"lala": "a"' in text
+                assert '"b": "4"' in text
+                assert '"d": "3"' in text
             remove(f)
             assert fpath == abspath(join(tmpdir, filename))
             f = downloader.download_file('%s?id=3&lala=b' % postfixtureurl, post=True,
@@ -163,10 +199,10 @@ class TestDownloader:
             fpath = abspath(f)
             with open(fpath, 'rt') as fi:
                 text = fi.read()
-                assert '"id":"3"' in text
-                assert '"lala":"b"' in text
-                assert '"a":"3"' in text
-                assert '"c":"2"' in text
+                assert '"id": "3"' in text
+                assert '"lala": "b"' in text
+                assert '"a": "3"' in text
+                assert '"c": "2"' in text
             remove(f)
             assert fpath == abspath(join(tmpdir, filename))
 
@@ -178,12 +214,12 @@ class TestDownloader:
         with Download() as downloader:
             result = downloader.download(fixtureurl)
             assert result.headers['Content-Length'] == '728'
-            result = downloader.download('%s?id=10&lala=a' % getfixtureurl, post=False,
-                                         parameters=OrderedDict([('b', '4'), ('d', '3')]))
-            assert result.json()['args'] == {'id': '10', 'lala': 'a', 'b': '4', 'd': '3'}
-            result = downloader.download('%s?id=3&lala=b' % postfixtureurl, post=True,
+            downloader.download('%s?id=10&lala=a' % getfixtureurl, post=False,
+                                parameters=OrderedDict([('b', '4'), ('d', '3')]))
+            assert downloader.get_json()['args'] == OrderedDict([('b', '4'), ('d', '3'), ('id', '10'), ('lala', 'a')])
+            downloader.download('%s?id=3&lala=b' % postfixtureurl, post=True,
                                          parameters=OrderedDict([('a', '3'), ('c', '2')]))
-            assert result.json()['form'] == {'id': '3', 'lala': 'b', 'a': '3', 'c': '2'}
+            assert downloader.get_json()['form'] == OrderedDict([('a', '3'), ('c', '2'), ('id', '3'), ('lala', 'b')])
 
     def test_download_tabular_key_value(self, fixtureurl, fixtureprocessurl):
         with Download() as downloader:
@@ -194,22 +230,32 @@ class TestDownloader:
             with pytest.raises(DownloadError):
                 downloader.download_tabular_key_value('NOTEXIST://NOTEXIST.csv')
 
+    @staticmethod
+    def fix_strings(result):  # This isn't needed when I run locally but is when run in Travis!
+        for x in result:
+            for y in result[x]:
+                result[x][y] = result[x][y].replace("'", '')
+
     def test_download_tabular_rows_as_dicts(self, fixtureprocessurl):
         with Download() as downloader:
             result = downloader.download_tabular_rows_as_dicts(fixtureprocessurl, headers=2)
-            assert result == {'coal': {'header2': '3', 'header3': '7.4', 'header4': "'needed'"},
-                              'gas': {'header2': '2', 'header3': '6.5', 'header4': "'n/a'"}}
+            self.fix_strings(result)
+            assert result == {'coal': {'header2': '3', 'header3': '7.4', 'header4': 'needed'},
+                              'gas': {'header2': '2', 'header3': '6.5', 'header4': 'n/a'}}
             result = downloader.download_tabular_rows_as_dicts(fixtureprocessurl, headers=2, keycolumn=2)
-            assert result == {'2': {'header1': 'gas', 'header3': '6.5', 'header4': "'n/a'"},
-                              '3': {'header1': 'coal', 'header3': '7.4', 'header4': "'needed'"}}
+            self.fix_strings(result)
+            assert result == {'2': {'header1': 'gas', 'header3': '6.5', 'header4': 'n/a'},
+                              '3': {'header1': 'coal', 'header3': '7.4', 'header4': 'needed'}}
 
     def test_download_tabular_cols_as_dicts(self, fixtureprocessurl):
         with Download() as downloader:
             result = downloader.download_tabular_cols_as_dicts(fixtureprocessurl, headers=2)
+            self.fix_strings(result)
             assert result == {'header2': {'coal': '3', 'gas': '2'},
                               'header3': {'coal': '7.4', 'gas': '6.5'},
-                              'header4': {'coal': "'needed'", 'gas': "'n/a'"}}
+                              'header4': {'coal': 'needed', 'gas': 'n/a'}}
             result = downloader.download_tabular_cols_as_dicts(fixtureprocessurl, headers=2, keycolumn=2)
+            self.fix_strings(result)
             assert result == {'header1': {'3': 'coal', '2': 'gas'},
                               'header3': {'3': '7.4', '2': '6.5'},
-                              'header4': {'3': "'needed'", '2': "'n/a'"}}
+                              'header4': {'3': 'needed', '2': 'n/a'}}
